@@ -1,7 +1,5 @@
-use std::os::unix::prelude::*;
-
 use super::prelude::*;
-use crate::common::{bytes2wcstring, get_program_name};
+use crate::common::{bytes2wcstring, get_program_name, osstr2wcstring, str2wcstring};
 use crate::env::config_paths::get_fish_path;
 use crate::future_feature_flags::{self as features, feature_test};
 use crate::proc::{
@@ -11,7 +9,7 @@ use crate::reader::reader_in_interactive_read;
 use crate::tty_handoff::{TERMINAL_OS_NAME, get_scroll_content_up_capability, xtversion};
 use crate::wutil::{Error, waccess, wbasename, wdirname, wrealpath};
 use cfg_if::cfg_if;
-use libc::F_OK;
+use nix::unistd::AccessFlags;
 use rust_embed::RustEmbed;
 
 /// Create an enum with name `$name`.
@@ -105,7 +103,7 @@ impl StatusCmdOpts {
     fn try_set_status_cmd(&mut self, subcmd: StatusCmd, streams: &mut IoStreams) -> bool {
         match self.status_cmd.replace(subcmd) {
             Some(existing) => {
-                streams.err.append(&wgettext_fmt!(
+                streams.err.appendln(&wgettext_fmt!(
                     BUILTIN_ERR_COMBO2_EXCLUSIVE,
                     "status",
                     existing.to_wstr(),
@@ -170,6 +168,10 @@ const LONG_OPTIONS: &[WOption] = &[
     wopt(L!("print-stack-trace"), NoArgument, 't'),
 ];
 
+localizable_consts! {
+    BUILTIN_INVALID_JOB_CONTROL_MODE "%s: Invalid job control mode '%s'"
+}
+
 /// Print the features and their values.
 fn print_features(streams: &mut IoStreams) {
     // TODO: move this to features.rs
@@ -211,8 +213,8 @@ fn parse_cmd_opts(
                     match fish_wcstoi(arg) {
                         Ok(level) if level >= 0 => level,
                         Err(Error::Overflow) | Ok(_) => {
-                            streams.err.append(&wgettext_fmt!(
-                                "%s: Invalid level value '%s'\n",
+                            streams.err.appendln(&wgettext_fmt!(
+                                "%s: Invalid level value '%s'",
                                 cmd,
                                 arg
                             ));
@@ -221,7 +223,7 @@ fn parse_cmd_opts(
                         _ => {
                             streams
                                 .err
-                                .append(&wgettext_fmt!(BUILTIN_ERR_NOT_NUMBER, cmd, arg));
+                                .appendln(&wgettext_fmt!(BUILTIN_ERR_NOT_NUMBER, cmd, arg));
                             return Err(STATUS_INVALID_ARGS);
                         }
                     }
@@ -247,8 +249,8 @@ fn parse_cmd_opts(
                     return Err(STATUS_CMD_ERROR);
                 }
                 let Ok(job_mode) = w.woptarg.unwrap().try_into() else {
-                    streams.err.append(&wgettext_fmt!(
-                        "%s: Invalid job control mode '%s'\n",
+                    streams.err.appendln(&wgettext_fmt!(
+                        BUILTIN_INVALID_JOB_CONTROL_MODE,
                         cmd,
                         w.woptarg.unwrap()
                     ));
@@ -308,23 +310,15 @@ impl RustEmbed for EmptyEmbed {
     fn get(_file_path: &str) -> Option<rust_embed::EmbeddedFile> {
         None
     }
-    fn iter() -> rust_embed::Filenames {
-        use rust_embed::Filenames::*;
-        cfg_if! {
-            // TODO This is a clone of rebuild_if_embedded_path_changed.
-            if #[cfg(any(not(debug_assertions), windows))] {
-                Embedded([].iter())
-            } else {
-                Dynamic(Box::new(None.into_iter()))
-            }
-        }
+    fn iter() -> impl Iterator<Item = std::borrow::Cow<'static, str>> {
+        std::iter::empty()
     }
 }
 
 cfg_if!(
     if #[cfg(feature = "embed-manpages")] {
         #[derive(RustEmbed)]
-        #[folder = "$FISH_RESOLVED_BUILD_DIR/fish-man/man1"]
+        #[folder = "$FISH_RESOLVED_BUILD_DIR/fish-docs/man/man1"]
         #[prefix = "man/man1/"]
         struct Docs;
     } else {
@@ -369,7 +363,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
             None => {
                 streams
                     .err
-                    .append(&wgettext_fmt!(BUILTIN_ERR_INVALID_SUBCMD, cmd, args[1]));
+                    .appendln(&wgettext_fmt!(BUILTIN_ERR_INVALID_SUBCMD, cmd, args[1]));
                 return Err(STATUS_INVALID_ARGS);
             }
         }
@@ -381,9 +375,9 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
         debug_assert!(args.is_empty(), "passed arguments to nothing");
 
         if get_login() {
-            streams.out.append(wgettext!("This is a login shell\n"));
+            streams.out.appendln(wgettext!("This is a login shell"));
         } else {
-            streams.out.append(wgettext!("This is not a login shell\n"));
+            streams.out.appendln(wgettext!("This is not a login shell"));
         }
         let job_control_mode = match get_job_control_mode() {
             JobControl::Interactive => wgettext!("Only on interactive jobs"),
@@ -392,7 +386,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
         };
         streams
             .out
-            .append(&wgettext_fmt!("Job control: %s\n", job_control_mode));
+            .appendln(&wgettext_fmt!("Job control: %s", job_control_mode));
         streams.out.append(&parser.stack_trace());
 
         return Ok(SUCCESS);
@@ -404,7 +398,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                 Some(j) => {
                     // Flag form used
                     if !args.is_empty() {
-                        streams.err.append(&wgettext_fmt!(
+                        streams.err.appendln(&wgettext_fmt!(
                             BUILTIN_ERR_ARG_COUNT2,
                             cmd,
                             c.to_wstr(),
@@ -417,7 +411,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                 }
                 None => {
                     if args.len() != 1 {
-                        streams.err.append(&wgettext_fmt!(
+                        streams.err.appendln(&wgettext_fmt!(
                             BUILTIN_ERR_ARG_COUNT2,
                             cmd,
                             c.to_wstr(),
@@ -427,8 +421,8 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                         return Err(STATUS_INVALID_ARGS);
                     }
                     let Ok(new_mode) = args[0].try_into() else {
-                        streams.err.append(&wgettext_fmt!(
-                            "%s: Invalid job control mode '%s'\n",
+                        streams.err.appendln(&wgettext_fmt!(
+                            BUILTIN_INVALID_JOB_CONTROL_MODE,
                             cmd,
                             args[0]
                         ));
@@ -442,7 +436,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
         STATUS_FEATURES => print_features(streams),
         c @ STATUS_TEST_FEATURE => {
             if args.len() != 1 {
-                streams.err.append(&wgettext_fmt!(
+                streams.err.appendln(&wgettext_fmt!(
                     BUILTIN_ERR_ARG_COUNT2,
                     cmd,
                     c.to_wstr(),
@@ -464,7 +458,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
         }
         c @ STATUS_GET_FILE => {
             if args.len() != 1 {
-                streams.err.append(&wgettext_fmt!(
+                streams.err.appendln(&wgettext_fmt!(
                     BUILTIN_ERR_ARG_COUNT2,
                     cmd,
                     c.to_wstr(),
@@ -519,7 +513,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                         invalid => {
                             streams
                                 .err
-                                .append(&wgettext_fmt!(BUILTIN_ERR_INVALID_SUBSUBCMD, cmd, subcmd.to_wstr(), invalid));
+                                .appendln(&wgettext_fmt!(BUILTIN_ERR_INVALID_SUBSUBCMD, cmd, subcmd.to_wstr(), invalid));
                             return Err(STATUS_INVALID_ARGS);
                         }
                     }
@@ -527,7 +521,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
             }
         }
         STATUS_LIST_FILES => {
-            use crate::util::wcsfilecmp_glob;
+            use fish_util::wcsfilecmp_glob;
             let mut paths = vec![];
             let mut add = |arg| {
                 let arg = crate::common::wcs2bytes(arg);
@@ -537,7 +531,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                     .chain(CMakeBinaryDir::iter())
                 {
                     if arg.is_empty() || path.starts_with(arg) {
-                        paths.push(bytes2wcstring(path.as_bytes()));
+                        paths.push(str2wcstring(&path));
                     }
                 }
             };
@@ -562,7 +556,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
         }
         c @ STATUS_TEST_TERMINAL_FEATURE => {
             if args.len() != 1 {
-                streams.err.append(&wgettext_fmt!(
+                streams.err.appendln(&wgettext_fmt!(
                     BUILTIN_ERR_ARG_COUNT2,
                     cmd,
                     c.to_wstr(),
@@ -572,14 +566,14 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                 return Err(STATUS_INVALID_ARGS);
             }
             if args[0] != "scroll-content-up" {
-                streams.err.appendln(wgettext_fmt!(
+                streams.err.appendln(&wgettext_fmt!(
                     "%s %s: unrecognized feature '%s'",
                     cmd,
                     c.to_wstr(),
                     args[0]
                 ));
                 return Err(STATUS_INVALID_ARGS);
-            };
+            }
             return if get_scroll_content_up_capability() == Some(true) {
                 Ok(SUCCESS)
             } else {
@@ -589,7 +583,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
 
         ref s => {
             if !args.is_empty() {
-                streams.err.append(&wgettext_fmt!(
+                streams.err.appendln(&wgettext_fmt!(
                     BUILTIN_ERR_ARG_COUNT2,
                     cmd,
                     s.to_wstr(),
@@ -600,26 +594,30 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
             }
             match s {
                 STATUS_BUILD_INFO => {
-                    let version = bytes2wcstring(crate::BUILD_VERSION.as_bytes());
-                    let target = bytes2wcstring(env!("BUILD_TARGET_TRIPLE").as_bytes());
-                    let host = bytes2wcstring(env!("BUILD_HOST_TRIPLE").as_bytes());
-                    let profile = bytes2wcstring(env!("BUILD_PROFILE").as_bytes());
+                    let version = str2wcstring(crate::BUILD_VERSION);
+                    let target = str2wcstring(env!("BUILD_TARGET_TRIPLE"));
+                    let host = str2wcstring(env!("BUILD_HOST_TRIPLE"));
+                    let profile = str2wcstring(env!("BUILD_PROFILE"));
                     streams.out.append(L!("Build system: "));
-                    let buildsystem = if cfg!(using_cmake) { "CMake" } else { "Cargo" };
-                    streams.out.appendln(bytes2wcstring(buildsystem.as_bytes()));
+                    let buildsystem = if cfg!(using_cmake) {
+                        L!("CMake")
+                    } else {
+                        L!("Cargo")
+                    };
+                    streams.out.appendln(buildsystem);
                     streams.out.append(L!("Version: "));
-                    streams.out.appendln(version);
+                    streams.out.appendln(&version);
                     if target == host {
                         streams.out.append(L!("Target (and host): "));
-                        streams.out.appendln(target);
+                        streams.out.appendln(&target);
                     } else {
                         streams.out.append(L!("Target: "));
-                        streams.out.appendln(target);
+                        streams.out.appendln(&target);
                         streams.out.append(L!("Host: "));
-                        streams.out.appendln(host);
+                        streams.out.appendln(&host);
                     }
                     streams.out.append(L!("Profile: "));
-                    streams.out.appendln(profile);
+                    streams.out.appendln(&profile);
                     streams.out.append(L!("Features: "));
                     let features: &[&str] = &[
                         #[cfg(feature = "embed-manpages")]
@@ -629,9 +627,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                         #[cfg(target_feature = "crt-static")]
                         "crt-static",
                     ];
-                    streams
-                        .out
-                        .appendln(bytes2wcstring(features.join(" ").as_bytes()));
+                    streams.out.appendln(&str2wcstring(features.join(" ")));
                     streams.out.appendln("");
                     return Ok(SUCCESS);
                 }
@@ -651,7 +647,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                         Some(f) => f,
                         None => wgettext!("Not a function").to_owned(),
                     };
-                    streams.out.appendln(f);
+                    streams.out.appendln(&f);
                 }
                 STATUS_LINE_NUMBER => {
                     // TBD is how to interpret the level argument when fetching the line number.
@@ -659,7 +655,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                     // streams.out.append_format(L"%d\n", parser.get_lineno(opts.level));
                     streams
                         .out
-                        .appendln(parser.get_lineno_for_display().to_wstring());
+                        .appendln(&parser.get_lineno_for_display().to_wstring());
                 }
                 STATUS_IS_INTERACTIVE => {
                     if is_interactive_session() {
@@ -736,16 +732,15 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                 }
                 STATUS_CURRENT_COMMANDLINE => {
                     let commandline = &parser.libdata().status_vars.commandline;
-                    streams.out.append(commandline);
-                    streams.out.append_char('\n');
+                    streams.out.appendln(commandline);
                 }
                 STATUS_FISH_PATH => {
                     use crate::env::config_paths::FishPath::*;
                     let result = match get_fish_path() {
                         Absolute(path) => {
-                            let path = bytes2wcstring(path.as_os_str().as_bytes());
+                            let path = osstr2wcstring(path);
                             Cow::Owned(match wrealpath(&path) {
-                                Some(p) if waccess(&p, F_OK) == 0 => p,
+                                Some(p) if waccess(&p, AccessFlags::F_OK).is_ok() => p,
                                 // realpath did not work, just append the path
                                 // - maybe this was obtained via $PATH?
                                 _ => path,
@@ -753,7 +748,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                         }
                         LookUpInPath => Cow::Borrowed(get_program_name()),
                     };
-                    streams.out.appendln(result);
+                    streams.out.appendln(&result);
                 }
                 STATUS_TERMINAL => {
                     let xtversion = xtversion().unwrap_or_default();
@@ -776,7 +771,7 @@ pub fn status(parser: &Parser, streams: &mut IoStreams, args: &mut [&wstr]) -> B
                 }
             }
         }
-    };
+    }
 
     Ok(SUCCESS)
 }
