@@ -30,12 +30,13 @@ use crate::parse_constants::{
     ParseError, ParseErrorCode, ParseErrorList, ParseKeyword, ParseTokenType, StatementDecoration,
     parse_error_offset_source_start,
 };
-use crate::parse_tree::{LineCounter, NodeRef, ParsedSourceRef};
+use crate::parse_tree::{NodeRef, ParsedSourceRef};
 use crate::parse_util::{
-    MaybeParentheses::CommandSubstitution, parse_util_locate_cmdsubst_range,
-    parse_util_unescape_wildcards,
+    MaybeParentheses::CommandSubstitution, locate_cmdsubst_range, unescape_wildcards,
+};
+use crate::parser::{
+    Block, BlockData, BlockId, BlockType, LoopStatus, Parser, ParserEnvSetMode, ProfileItem,
 };
-use crate::parser::{Block, BlockData, BlockId, BlockType, LoopStatus, Parser, ProfileItem};
 use crate::parser_keywords::parser_keywords_is_subcommand;
 use crate::path::{path_as_implicit_cd, path_try_get_path};
 use crate::prelude::*;
@@ -50,7 +51,8 @@ use crate::timer::push_timer;
 use crate::tokenizer::{PipeOrRedir, TokenType, variable_assignment_equals_pos};
 use crate::trace::{trace_if_enabled, trace_if_enabled_with_args};
 use crate::wildcard::wildcard_match;
-use fish_wchar::WExt;
+use fish_common::help_section;
+use fish_widestring::WExt;
 use libc::{ENOTDIR, EXIT_SUCCESS, STDERR_FILENO, STDOUT_FILENO, c_int};
 use std::io::ErrorKind;
 use std::rc::Rc;
@@ -59,7 +61,7 @@ use std::sync::Arc;
 /// An eval_result represents evaluation errors including wildcards which failed to match, syntax
 /// errors, or other expansion errors. It also tracks when evaluation was skipped due to signal
 /// cancellation. Note it does not track the exit status of commands.
-#[derive(Eq, PartialEq)]
+#[derive(Debug, Eq, PartialEq)]
 pub enum EndExecutionReason {
     /// Evaluation was successful.
     Ok,
@@ -82,9 +84,9 @@ pub struct ExecutionContext<'a> {
     // unwinding.
     cancel_signal: Option<Signal>,
 
-    // Helper to count lines.
+    // The currently executing pipeline node.
     // This is shared with the Parser so that the Parser can access the current line.
-    line_counter: &'a ScopedRefCell<LineCounter<ast::JobPipeline>>,
+    pipeline_node: &'a ScopedRefCell<Option<NodeRef<ast::JobPipeline>>>,
 
     /// The block IO chain.
     /// For example, in `begin; foo ; end < file.txt` this would have the 'file.txt' IO.
@@ -115,12 +117,14 @@ macro_rules! report_error_formatted {
 }
 
 pub fn varname_error(command: &wstr, bad_name: &wstr) -> WString {
-    wgettext_fmt!(
+    let mut e = wgettext_fmt!(
         BUILTIN_ERR_VARNAME,
         command,
         bad_name,
         help_section!("language#shell-variable-and-function-names")
-    )
+    );
+    e.push('\n');
+    e
 }
 
 impl<'a> ExecutionContext<'a> {
@@ -129,13 +133,13 @@ impl<'a> ExecutionContext<'a> {
     pub fn new(
         pstree: ParsedSourceRef,
         block_io: IoChain,
-        line_counter: &'a ScopedRefCell<LineCounter<ast::JobPipeline>>,
+        pipeline_node: &'a ScopedRefCell<Option<NodeRef<ast::JobPipeline>>>,
         test_only_suppress_stderr: bool,
     ) -> Self {
         Self {
             pstree,
             cancel_signal: None,
-            line_counter,
+            pipeline_node,
             block_io,
             test_only_suppress_stderr,
         }
@@ -451,7 +455,7 @@ impl<'a> ExecutionContext<'a> {
             });
 
         if infinite_recursive_statement.is_some() {
-            *out_func_name = forbidden_function_name.to_owned();
+            forbidden_function_name.clone_into(out_func_name);
         }
 
         // may be none
@@ -648,8 +652,11 @@ impl<'a> ExecutionContext<'a> {
                     vals.clone(),
                 ));
             }
-            ctx.parser()
-                .set_var_and_fire(variable_name, EnvMode::LOCAL | EnvMode::EXPORT, vals);
+            ctx.parser().set_var_and_fire(
+                variable_name,
+                ParserEnvSetMode::new(EnvMode::LOCAL | EnvMode::EXPORT),
+                vals,
+            );
         }
         EndExecutionReason::Ok
     }
@@ -774,7 +781,7 @@ impl<'a> ExecutionContext<'a> {
                         std::io::Error::from_raw_os_error(external_cmd.err.unwrap().into()),
                     );
                 }
-            };
+            }
             path
         } else {
             WString::new()
@@ -926,12 +933,12 @@ impl<'a> ExecutionContext<'a> {
             );
         }
 
-        let retval = ctx.parser().vars().set(
+        let retval = ctx.parser().set_var(
             &for_var_name,
-            EnvMode::LOCAL | EnvMode::USER,
+            ParserEnvSetMode::user(EnvMode::LOCAL),
             var.map_or(vec![], |var| var.as_list().to_owned()),
         );
-        assert!(retval == EnvStackSetResult::Ok);
+        assert_eq!(retval, EnvStackSetResult::Ok);
 
         trace_if_enabled_with_args(ctx.parser(), L!("for"), &arguments);
 
@@ -946,12 +953,14 @@ impl<'a> ExecutionContext<'a> {
                 break;
             }
 
-            let retval = ctx
-                .parser()
-                .vars()
-                .set(&for_var_name, EnvMode::USER, vec![val]);
-            assert!(
-                retval == EnvStackSetResult::Ok,
+            let retval = ctx.parser().set_var(
+                &for_var_name,
+                ParserEnvSetMode::user(EnvMode::empty()),
+                vec![val],
+            );
+            assert_eq!(
+                retval,
+                EnvStackSetResult::Ok,
                 "for loop variable should have been successfully set"
             );
             event::fire(ctx.parser(), evt.clone());
@@ -1099,13 +1108,17 @@ impl<'a> ExecutionContext<'a> {
             }
             ExpandResultCode::ok => {
                 if switch_values_expanded.len() > 1 {
-                    return report_error!(
+                    let mut error = wgettext_fmt!(
+                        "switch: Expected at most one argument, got %u",
+                        switch_values_expanded.len()
+                    );
+                    error.push('\n');
+                    return report_error_formatted!(
                         self,
                         ctx,
                         STATUS_INVALID_ARGS,
                         &statement.argument,
-                        "switch: Expected at most one argument, got %u\n",
-                        switch_values_expanded.len()
+                        error
                     );
                 }
             }
@@ -1149,7 +1162,7 @@ impl<'a> ExecutionContext<'a> {
             if case_result == EndExecutionReason::Ok {
                 for arg in case_args {
                     // Unescape wildcards so they can be expanded again.
-                    let unescaped_arg = parse_util_unescape_wildcards(&arg);
+                    let unescaped_arg = unescape_wildcards(&arg);
                     if wildcard_match(&switch_value_expanded, &unescaped_arg, false) {
                         // If this matched, we're done.
                         matching_case_item = Some(case_item);
@@ -1164,7 +1177,7 @@ impl<'a> ExecutionContext<'a> {
 
         if let Some(case_item) = matching_case_item {
             // Success, evaluate the job list.
-            assert!(result == EndExecutionReason::Ok, "Expected success");
+            assert_eq!(result, EndExecutionReason::Ok, "Expected success");
             result = self.run_job_list(ctx, &case_item.body, Some(sb));
         }
 
@@ -1456,7 +1469,7 @@ impl<'a> ExecutionContext<'a> {
                 if oper.mode == RedirectionMode::Input && {
                     let redir_unexpanded = self.node_source(redir_node);
                     redir_unexpanded.starts_with(L!("<("))
-                        && match parse_util_locate_cmdsubst_range(
+                        && match locate_cmdsubst_range(
                             &redir_unexpanded[1..],
                             &mut 0,
                             false,
@@ -1523,9 +1536,8 @@ impl<'a> ExecutionContext<'a> {
         let _saved_eval_level = ctx.parser().push_scope(|s| s.eval_level += 1);
 
         // Save the executing node.
-        let _saved_node = self
-            .line_counter
-            .scoped_set(std::ptr::from_ref(job_node), |s| &mut s.node);
+        let executing_node = NodeRef::new(Arc::clone(self.pstree()), job_node);
+        let _saved_node = self.pipeline_node.scoped_replace(Some(executing_node));
 
         // Profiling support.
         let profile_item_id = ctx.parser().create_profile_item();
@@ -1859,7 +1871,7 @@ impl<'a> ExecutionContext<'a> {
             .is_some_and(|job_group| job_group.has_job_id() || !j.wants_job_id())
             && !j.is_initially_background()
         {
-            j.group = ctx.job_group.clone();
+            j.group.clone_from(&ctx.job_group);
             return;
         }
 
